@@ -44,6 +44,7 @@
 #include "gedit-mdi.h"
 #include "gedit-recent.h" 
 #include "gedit-file-selector-util.h"
+#include "gedit-io-error-dialogs.h"
 #include "gedit-plugins-engine.h"
 #include "recent-files/egg-recent-model.h"
 #include "gedit-prefs-manager-app.h"
@@ -74,7 +75,8 @@ static gchar* gedit_default_path = NULL;
 /* type of the elements of the uris_to_open list */
 typedef struct {
 	gchar *uri;
-	gboolean create;
+	gint create: 1;
+	gint first: 1;
 } UriToLoad;
 
 /* Global variables for managing async loading */
@@ -86,6 +88,8 @@ static gint                 line 		= -1;
 static GSList              *new_children 	= NULL;
 static GSList		   *children_to_unref	= NULL;
 static gint		    times_called	= 0;
+static GeditDocument	   *first_document	= NULL;
+static gboolean		    active_doc_reused   = FALSE;
 
 static GtkWidget           *loading_window	= NULL;
 static GtkWidget           *progress_bar	= NULL;
@@ -266,8 +270,9 @@ gedit_file_save (GeditMDIChild* child, gboolean force)
 			bonobo_mdi_set_active_view (BONOBO_MDI (gedit_mdi), view);
 		}
 
-		gedit_utils_error_reporting_saving_file (uri, error,
-					GTK_WINDOW (gedit_get_active_window ()));
+		gedit_error_reporting_saving_file (uri,
+						   error,
+						   GTK_WINDOW (gedit_get_active_window ()));
 
 		g_error_free (error);
 
@@ -395,8 +400,9 @@ gedit_file_save_as (GeditMDIChild *child)
 		{
 			g_return_val_if_fail (error != NULL, FALSE);
 
-			gedit_utils_error_reporting_saving_file (uri, error,
-						GTK_WINDOW (gedit_get_active_window ()));
+			gedit_error_reporting_saving_file (uri,
+							   error,
+							   GTK_WINDOW (gedit_get_active_window ()));
 
 			g_error_free (error);
 
@@ -589,7 +595,7 @@ gedit_file_revert_dialog (GeditDocument *doc)
 		hours = seconds / 3600;
 
 		secondary_msg = g_strdup_printf (
-					ngettext ("Changes made to the document in the last %d hour "
+					ngettext ("Changes made to the document in the last hour "
 					    	  "will be permanently lost.",
 						  "Changes made to the document in the last %d hours "
 					    	  "will be permanently lost.",
@@ -601,9 +607,9 @@ gedit_file_revert_dialog (GeditDocument *doc)
 					 GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 					 GTK_MESSAGE_QUESTION,
 					 GTK_BUTTONS_NONE,
-					 primary_msg);
+					 "%s", primary_msg);
 	gtk_message_dialog_format_secondary_text (GTK_MESSAGE_DIALOG (dialog),
-						  secondary_msg);
+						  "%s", secondary_msg);
 	g_free (primary_msg);
 	g_free (secondary_msg);
 
@@ -659,8 +665,9 @@ document_reverted_cb (GeditDocument *document,
 
 		gedit_debug (DEBUG_DOCUMENT, "Error reverting file %s", uri_to_display);
 
-		gedit_utils_error_reporting_reverting_file (raw_uri, error,
-					GTK_WINDOW (gedit_get_active_window ()));
+		gedit_error_reporting_reverting_file (raw_uri,
+						      error,
+						      GTK_WINDOW (gedit_get_active_window ()));
 
 		gedit_utils_flash_va (_("The document \"%s\" has not been reverted."), uri_to_display);
 	}
@@ -824,7 +831,7 @@ gedit_file_open_from_stdin (GeditMDIChild *active_child)
 				GTK_DIALOG_MODAL | GTK_DIALOG_DESTROY_WITH_PARENT,
 			   	GTK_MESSAGE_ERROR,
 			   	GTK_BUTTONS_OK,
-				errstr);
+				"%s", errstr);
 
 		gtk_dialog_set_default_response (GTK_DIALOG (dialog), GTK_RESPONSE_OK);
 
@@ -1090,13 +1097,13 @@ document_loaded_cb (GeditDocument *document,
 
 		gedit_recent_remove (uri);
 
-		gedit_utils_error_reporting_loading_file (uri, encoding_to_use, (GError *)error,
-					GTK_WINDOW (gedit_get_active_window ()));
+		gedit_error_reporting_loading_file (uri,
+						    encoding_to_use,
+						    (GError *)error,
+						    GTK_WINDOW (gedit_get_active_window ()));
 
 		if (new_child != NULL)
-		{
 			children_to_unref = g_slist_prepend (children_to_unref, new_child);
-		}
 
 		gedit_debug (DEBUG_DOCUMENT, "Returning %s", uri_to_display);
 	}
@@ -1107,8 +1114,6 @@ document_loaded_cb (GeditDocument *document,
 		gedit_utils_set_status (NULL);
 		gedit_utils_flash_va (_("Loaded file \"%s\""), uri_to_display);
 
-		gedit_document_goto_line (document, MAX (0, line - 1));
-
 		if (new_child != NULL)
 		{
 			bonobo_mdi_add_child (BONOBO_MDI (gedit_mdi), BONOBO_MDI_CHILD (new_child));
@@ -1178,12 +1183,31 @@ open_files_done ()
 	/* Add all the views in the right order */
 	new_children = g_slist_reverse (new_children);
 
+	if (first_document != NULL)
+		gedit_document_goto_line (first_document, MAX (0, line - 1));	
+	
 	PROFILE (
 		g_message ("Document Loaded: %.3f", g_timer_elapsed (timer, NULL));
 	)
 
 	bonobo_mdi_add_views (BONOBO_MDI (gedit_mdi),
-			      new_children);
+			      new_children,
+			      !active_doc_reused);
+
+	if (active_doc_reused)
+	{
+		GeditView *view = GEDIT_VIEW (gedit_get_active_view ());
+		GtkTextBuffer *buffer;
+
+		buffer = gtk_text_view_get_buffer (gedit_view_get_gtk_text_view (view));
+
+		gtk_text_view_scroll_to_mark (gedit_view_get_gtk_text_view (view),
+				      gtk_text_buffer_get_insert (buffer),
+				      0.25,
+				      FALSE,
+				      0.0,
+				      0.0);
+	}
 
 	PROFILE (
 		g_message ("View added: %.3f", g_timer_elapsed (timer, NULL));
@@ -1211,6 +1235,8 @@ open_files_done ()
 	opened_uris = 0;
 	encoding_to_use = NULL;
 	line = -1;
+	first_document = NULL;
+	active_doc_reused = FALSE;
 
 	PROFILE (
 		g_message ("Done all: %.3f", g_timer_elapsed (timer, NULL));
@@ -1301,8 +1327,10 @@ open_files ()
 		if (new_child == NULL)
 		{
 			/* FIXME: this is a too generic error message - Paolo */
-			gedit_utils_error_reporting_loading_file (uri, encoding_to_use, NULL,
-					GTK_WINDOW (gedit_get_active_window ()));
+			gedit_error_reporting_loading_file (uri,
+							    encoding_to_use,
+							    NULL,
+							    GTK_WINDOW (gedit_get_active_window ()));
 
 			gedit_utils_flash_va (_("Error loading file \"%s\""), uri_to_display);
 
@@ -1316,6 +1344,9 @@ open_files ()
 			return;
 		}
 
+		if (utl->first)
+			first_document = new_child->document;
+
 		g_signal_connect (G_OBJECT (new_child->document),
 				  "loading",
 				  G_CALLBACK (document_loading_cb),
@@ -1330,6 +1361,12 @@ open_files ()
 	}
 	else
 	{	
+		if (utl->first)
+		{
+			first_document = active_document;
+			active_doc_reused = TRUE;
+		}
+
 		PROFILE (
 			g_message ("Load file: %.3f", g_timer_elapsed (timer, NULL));
 		)
@@ -1349,7 +1386,6 @@ open_files ()
 		PROFILE (
 			g_message ("Start loading: %.3f", g_timer_elapsed (timer, NULL));
 		)
-
 	}
 	
 	g_free (uri_to_display);
@@ -1405,6 +1441,7 @@ gedit_file_open_uri_list (GSList *uri_list,
 		u = g_new (UriToLoad, 1);
 
 		u->uri = uri;
+		u->first = FALSE;
 
 		if (create &&
 		    gedit_utils_uri_has_file_scheme (uri) &&
@@ -1415,6 +1452,9 @@ gedit_file_open_uri_list (GSList *uri_list,
 		else
 		{
 			u->create = FALSE;
+
+			if (l == uri_list)
+				u->first = TRUE;
 		}
 
 		uris_to_open = g_slist_prepend (uris_to_open, u);
